"""
OXASL - Module to generate a suitable mask for ASL data

Copyright (c) 2008 University of Nottingham
"""
import numpy as np
import scipy as sp

from fsl.data.image import Image

from oxasl import reg
from oxasl.reporting import LightboxImage

def generate_mask(wsp):
    """
    For compatibility
    """
    run(wsp)

def run(wsp):
    """
    Generate mask for ASL data

    - If a ready-made mask image is provided or has already been generated, this is returned
    - If a structural image is provided this will be used. Brain extraction and registration
      will be performed if required
    - If a calibration image is provided, this is used. It is assumed to be in the same space
      as the ASL data
    - If none of the above are present, the ASL data itself is averaged and brain extracted
      to produce the mask

    Required workspace attributes
    -----------------------------

    Formally there are no required attributes, however at least one image must be provided
    which enables a mask to be generated.

    Optional workspace attributes
    -----------------------------

     - ``asldata`` : ASL data image
     - ``mask``    : Existing brain mask
     - ``struc``   : Structural image (wholehead)
     - ``struc_brain``: Already brain-extracted structural image
     - ``asl2struc`` : Existring ASL->Structural space transformation matrix
     - ``calib``   : Calibration image
     - ``aslref`` : ASL registration source image
    """
    if wsp.rois is not None and wsp.rois.mask is not None:
        return

    wsp.sub("rois")
    wsp.log.write("\nGenerating ASL data mask\n")

    # Reporting
    page = wsp.report.page("mask")
    page.heading("Mask generation", level=0)

    if wsp.mask is not None:
        if reg.get_img_space(wsp, wsp.mask) != "asl":
            raise ValueError("Mask supplied is not in ASL space: %s" % wsp.mask.name)
        wsp.rois.mask_src = "user"
        mask_source = "ASL space mask provided by user: %s" % wsp.mask.name
        wsp.rois.mask = wsp.mask
    elif wsp.structural is not None and wsp.structural.struc is not None:
        # Preferred option is to use brain extracted structural
        wsp.rois.mask_src = "struc"
        page.heading("Brain extracted structural image", level=1)
        page.image("struc_brain", LightboxImage(wsp.structural.brain, bgimage=wsp.structural.struc))
        wsp.rois.mask_struc = wsp.structural.brain_mask
        wsp.rois.mask_asl = reg.change_space(wsp, wsp.structural.brain_mask, "asl")
        wsp.rois.mask = Image(sp.ndimage.morphology.binary_fill_holes((wsp.rois.mask_asl.data > 0.25)).astype(np.int32), header=wsp.rois.mask_asl.header)
        mask_source = "generated from brain extracting structural image and registering to ASL space"
    else:
        # Alternatively, use registration image (which will be BETed calibration or mean ASL image)
        wsp.rois.mask_src = "aslref"
        wsp.rois.mask = Image((wsp.reg.aslref.data != 0).astype(np.int32), header=wsp.reg.aslref.header)
        mask_source = "generated from brain extracted registration ASL image"

    wsp.log.write(" - Mask %s\n" % mask_source)

    page.heading("Masked ASL brain image", level=1)
    page.text("Mask was %s" % mask_source)
    page.text("PW ASL image masked by ASL-space mask")

    if wsp.asldata.iaf in ("diff", "tc", "ct"):
        page.image("mask_outline", LightboxImage(wsp.rois.mask, bgimage=wsp.asldata.perf_weighted(), outline=True))
    else:
        page.image("mask_outline", LightboxImage(wsp.rois.mask, bgimage=wsp.asldata.mean(), outline=True))



"""
OXASL - Module to generate additional ROIs for analysis

Copyright (c) 2008 University of Nottingham
"""
import numpy as np

from fsl.data.image import Image
from fsl.data.atlases import AtlasRegistry

from oxasl import reg

def run(wsp):
    """
    Generate additional ROIs used for reporting and region analysis

    wsp.rois.gm_asl: GM mask in ASL space (>0.5 probability)
    wsp.rois.wm_asl: WM mask in ASL space (>0.5 probability)
    wsp.rois.pure_gm_asl: 'pure' GM mask in ASL space (default > 0.8 probability, but uses pure_gm_thresh option)
    wsp.rois.pure_wm_asl: 'pure' WM mask in ASL space (default > 0.9 probability, but uses pure_gm_thresh option)
    wsp.rois.cortial_gm_asl: Cortical GM mask in ASL space, same as pure_gm_asl but masked to remove subcortical structures
    wsp.rois.cerebral_wm_asl: Cererbral WM mask in ASL space, same as pure_wm_asl but masked to remove subcortical structures
    """
    if wsp.structural.struc is not None:
        # Get the cortex 
        atlases = AtlasRegistry()
        atlases.rescanAtlases()
        atlas = atlases.loadAtlas("harvardoxford-subcortical", loadSummary=False, resolution=2)
        wsp.rois.cortex_std = Image(np.mean(atlas.data[..., 0:2] + atlas.data[..., 11:13], axis=-1) * 2, header=atlas.header)
        cortex_asl = reg.change_space(wsp, wsp.rois.cortex_std, "asl")
        wsp.rois.cortex_asl = Image((cortex_asl.data > 50).astype(int), header=cortex_asl.header)

        if wsp.structural.gm_pv_asl is None:
            wsp.structural.gm_pv_asl = reg.change_space(wsp, wsp.structural.gm_pv, "asl")
            wsp.structural.wm_pv_asl = reg.change_space(wsp, wsp.structural.wm_pv, "asl")
            wsp.structural.csf_pv_asl = reg.change_space(wsp, wsp.structural.csf_pv, "asl")

        gm = np.asarray(wsp.structural.gm_pv_asl.data)
        wm = np.asarray(wsp.structural.wm_pv_asl.data)

        wsp.rois.pure_gm_thresh = wsp.ifnone("pure_gm_thresh", 0.8)
        wsp.rois.pure_wm_thresh = wsp.ifnone("pure_wm_thresh", 0.9)
        some_gm, some_wm = gm > 0.5, wm > 0.5
        pure_gm, pure_wm = gm > wsp.rois.pure_gm_thresh, wm > wsp.rois.pure_wm_thresh
        wsp.rois.gm_asl = Image(some_gm.astype(int), header=wsp.structural.gm_pv_asl.header)
        wsp.rois.pure_gm_asl = Image(pure_gm.astype(int), header=wsp.structural.gm_pv_asl.header)
        wsp.rois.wm_asl = Image(some_wm.astype(int), header=wsp.structural.wm_pv_asl.header)
        wsp.rois.pure_wm_asl = Image(pure_wm.astype(int), header=wsp.structural.wm_pv_asl.header)
        wsp.rois.cortical_gm_asl = Image(np.logical_and(pure_gm, cortex_asl.data).astype(int), header=wsp.structural.gm_pv_asl.header)
        wsp.rois.cerebral_wm_asl = Image(np.logical_and(pure_wm, cortex_asl.data).astype(int), header=wsp.structural.wm_pv_asl.header)
